%% Plot
% Setpoint chasing in surge and sway and user specified guidance system
% in yaw, with desired heading equal 0 degrees. P controller in yaw.
% Hs = 1.25 m
% Kp_x = 0.35, Kd_x = 9
% Kp_y = 0.23, Kd_y = 4
% Kp_psi = 0.2

%comma2point_overwrite('SP_Hs125.txt')

% Reading file
All_data = dlmread('SP_Hs125.txt');

% Dagens posisjon
X = 1490+11;
Y = -524;

x_m = (All_data(:,1)-X)/1000;
y_m = (All_data(:,2)-Y)/1000;
psi_m = All_data(:,3);

psi_error = All_data(:,4);
x_error = All_data(:,5);
y_error = All_data(:,6);

time_abs = All_data(:,7);
time = 0:0.01:((length(time_abs)*0.01)-0.01);

tau_com_psi = All_data(:,8);
tau_com_x = All_data(:,9);
tau_com_y = All_data(:,10);

tau_psi = All_data(:,11);
tau_surge = All_data(:,12);
tau_sway = All_data(:,13);

psi_ref = All_data(:,14);
x_ref = (All_data(:,15)-X)/1000;
y_ref = (All_data(:,16)-Y)/1000;

% Plot

width=7.8;
height=10;
f = figure('Units','centimeters','Position',[0,0,width,height],'PaperPositionMode','auto');
h = subplot(3,1,1);
plot(time,x_m,time,x_ref);
grid on;
legend('X_E','X_{Ed}');
ylabel('$X_E$ [m]','interpreter','latex');
axis([0 time(end) -0.2 0.2]);
%title('North position','Interpreter','latex')
set(gca, 'TickLabelInterpreter','latex');
h = subplot(3,1,2);
plot(time,y_m,time,y_ref);
grid on;
legend('Y_E','Y_{Ed}');
ylabel('$Y_E$ [m]','interpreter','latex');
axis([0 time(end) -0.2 0.2]);
%title('East position','Interpreter','latex')
set(gca, 'TickLabelInterpreter','latex');
h = subplot(3,1,3);
plot(time,psi_m,time,psi_ref);
grid on;
legend('\psi','\psi_{d}');
xlabel('Time [s]','interpreter','latex');
ylabel('$\psi$ [deg]','interpreter','latex');
axis([0 time(end) -20 20]);
%title('Heading','Interpreter','latex')
set(gca, 'TickLabelInterpreter','latex');
set(f, 'PaperSize',[width, height]);
print(f, 'Position_EX_SP_U_Hs125', '-dpdf');

% figure()
% plot(time, psi_m)
% hold on
% plot(time,psi_ref)
% axis([0 time(end) -20 20]) 
% title('Heading')
% xlabel('Time [sec]')
% ylabel('\psi [degree]')
% legend('\psi','\psi_r')
% 
% figure()
% plot(time, x_m)
% hold on
% plot(time, x_ref)
% axis([0 time(end) -0.2 0.2]) 
% title('Surge')
% xlabel('Time [sec]')
% ylabel('Surge [m]')
% legend('x','x_r')
% 
% figure()
% plot(time, y_m)
% hold on
% plot(time, y_ref)
% axis([0 time(end) -0.2 0.2])
% title('Sway')
% xlabel('Time [sec]')
% ylabel('Sway [m]')
% legend('y','y_r')


f2 = figure('Units','centimeters','Position',[0,0,width,height],'PaperPositionMode','auto');
h = subplot(3,1,1);
plot(time,tau_surge,time, tau_com_psi);
grid on;
axis([0 time(end) -0.05 0.05]);
legend('\tau_{c}','\tau_{com}');
ylabel('Surge [N]','interpreter','latex');
%title('Control force in surge','Interpreter','latex')
set(gca, 'TickLabelInterpreter','latex');
h = subplot(3,1,2);
plot(time,tau_sway,time, tau_com_y);
grid on;
axis([0 time(end) -0.05 0.05]);
ylabel('Sway [N]','interpreter','latex');
legend('\tau_{c}','\tau_{com}');
set(gca, 'TickLabelInterpreter','latex');
h = subplot(3,1,3);
plot(time,tau_psi,time, tau_com_psi);
axis([0 time(end) -0.05 0.05]);
grid on;
legend('\tau_{c}','\tau_{com}');
xlabel('Time [s]','interpreter','latex');
ylabel('Yaw [Nm]','interpreter','latex');
%title('Control moment in yaw','Interpreter','latex')
set(gca, 'TickLabelInterpreter','latex');
set(f2, 'PaperSize',[width, height]);
print(f2, 'Tau_EX_SP_U_Hs125', '-dpdf')




% figure()
% plot(time, tau_psi)
% hold on
% plot(time, tau_com_psi)
% axis([0 time(end) -0.1 0.1])
% title('Thrust heading')
% xlabel('Time [sec]')
% ylabel('Thrust [N/s]')
% legend('\tau_{PID}','\tau_{com}')
% 
% figure()
% plot(time, tau_surge)
% hold on
% plot(time, tau_com_x)
% axis([0 time(end) -0.1 0.1])
% title('Thrust surge')
% xlabel('Time [sec]')
% ylabel('Thrust [N]')
% legend('\tau_{PID}','\tau_{com}')
% 
% figure()
% plot(time, tau_sway)
% hold on
% plot(time, tau_com_y)
% axis([0 time(end) -0.1 0.1])
% title('Thrust sway')
% xlabel('Time [sec]')
% ylabel('Thrust [N]')
% legend('\tau_{PID}','\tau_{com}')
% 
% r = 0.1;
% k = 0;
% for theta = 0:pi/100:2*pi
%     k = k+1;
%     x(k) = r*cos(theta);
%     y(k) = r*sin(theta);
% end
% 
% figure()
% plot(x,y);
% hold on
% plot(y_m,x_m)
% axis equal
% title('North and East')
% xlabel('East [m]')
% ylabel('North [m]')

f2 = figure('Units','centimeters','Position',[0,0,width,height],'PaperPositionMode','auto');
h = subplot(3,1,1);
plot(time, x_error);
grid on;
axis([0 time(end) -0.03 0.03]);
ylabel('$Error_{x}$ [m]','interpreter','latex');
%title('Control force in surge','Interpreter','latex')
set(gca, 'TickLabelInterpreter','latex');
h = subplot(3,1,2);
plot(time, x_error);
grid on;
axis([0 time(end) -0.03 0.03]);
ylabel('$Error_{y}$ [m]','interpreter','latex');
set(gca, 'TickLabelInterpreter','latex');
h = subplot(3,1,3);
plot(time, psi_error);
axis([0 time(end) -10 10])
grid on;
xlabel('Time [s]','interpreter','latex');
ylabel('$Error_{\psi}$ [deg]','interpreter','latex');
set(gca, 'TickLabelInterpreter','latex');
set(f2, 'PaperSize',[width, height]);
print(f2, 'Error_EX_SP_U_Hs125', '-dpdf')


% figure()
% plot(time, x_error)
% axis([0 time(end) -0.03 0.03])
% title('Error in X position')
% xlabel('Time [sec]')
% ylabel('Error [m]')
% 
% figure()
% plot(time, y_error)
% axis([0 time(end) -0.03 0.03])
% title('Error in Y position')
% xlabel('Time [sec]')
% ylabel('Error [m]')
% 
% figure()
% plot(time, psi_error)
% axis([0 time(end) -10 10])
% title('Error in heading')
% xlabel('Time [sec]')
% ylabel('Error [degree]')