%% Startup file for CSAD_Thrust_simulation

%% Modifiable controls:
%tc = [0;0.5;0];         % Control thrust into thrust allocation
Thruster_lock = 1;      % if 0 --> Thrusters locked to predescribed angles, if 1 --> Azimuth thrusters
Thruster_control = 1;   % Chooses Controller: 1 = Speed , 2 = Torque , 3 = Power , 4 = Power & torque
%Thrust_algorithm = 1;   % Chooses which algorithm is used 1 = Quadratic solver , 2 = Pseudoinverse solver
%% Parameters

%Thruster coefficients. Found from Bollard pull

K_T1f = 0.3763;
K_T2f = 0.3901;
K_T3f = 0.3776;
K_T4f = 0.5641;
K_T5f = 0.4799;
K_T6f = 0.5588;
K_T1r = K_T1f;
K_T2r = K_T2f;
K_T3r = K_T3f;
K_T4r = K_T4f;
K_T5r = K_T5f;
K_T6r = K_T6f;

K_q1f = 0.0113;
K_q2f = 0.0117;
K_q3f = 0.0113;
K_q4f = 0.0169;
K_q5f = 0.0144;
K_q6f = 0.0168;
K_q1r = K_q1f;
K_q2r = K_q2f;
K_q3r = K_q3f;
K_q4r = K_q4f;
K_q5r = K_q5f;
K_q6r = K_q6f;

lambda = 90;        % Scaling parameter
eps = 1E-5;         % Constant for avoiding singularities

%% Model vessel parameters
Rho_m = 1000;     % Water density tank [Kg/m^3]
D = 3.0/100;      % Propeller diameter model vessel  [m] 
n_max = 64.6;     % Maximum propeller speed model [RPM]               
T_max = 1.5034;   % Max produced thrust [N]
I_s = 0.05;       % Moment of inertia [kg*m^2]
Rho = Rho_m;
Max_rotation = 12*sqrt(lambda); % Max rotation of thrusters [1/s]
Max_thrust = 1.5034;  % maximum force for each thruster
Min_thrust = -0.85; % maximum negative thrust for each thruster
Max_Acceleration = 2; % Max propeller accelleration [1/s^2]
Max_thrust_rate = 0.0285;

Thruster.T1 = [ 96.1/90  0        180 2];    %[m,m,Deg,Deg/s]
Thruster.T2 = [ 84.1/90  9.9/90  -135 2];    %[m,m,Deg,Deg/s] 
Thruster.T3 = [ 84.1/90 -9.9/90   90  2];    %[m,m,Deg,Deg/s] 
Thruster.T4 = [-104.8/90 0        0   2];    %[m,m,Deg,Deg/s] 
Thruster.T5 = [-89.2/90 -14.9/90  45  2];    %[m,m,Deg,Deg/s] 
Thruster.T6 = [-89.2/90  14.9/90 -90  2];    %[m,m,Deg,Deg/s] 

% Thruster control
eps_c = 5;        % Constant for switching between positive and negative thrust
n_c = 5;          % Switching width between K_tc and K_tcr
omega_r0 = 0.90;  % Natural frequency [1/s] 
zeta_r = .78;     % Damping ratio [-]
H1 = tf(omega_r0^2,[1 2*zeta_r*omega_r0 omega_r0^2]);
hd = c2d(H1,0.1,'foh');

% Transfer function for Engine
H2 = tf(1,[0.02 1]);
hd2 = c2d(H2,0.01,'zoh');

Kp = 0.3;         % Core controller gain [-]
nd_slew = 0.5*90; % Max RPS rate [1/s^2]
K_omega = 0.3;    % Linear friction coefficient [-] 0.3
epsilon = 0.5;    % Constant for friction component
Q_f0 = 0.3;       % Friction constant

% Core thruster parameters for combined torque and power
k_cc = 1;
p_cc = 0.5;
r_cc = 4;


% constraints on thrusters:
%ABS GUIDE Table
x = [ 1 2 3 4 5 6 7 8 9 10 11 12 13 14 15];
v = [ 30 26.3 22.8 20.6 19 17.8 16.8 16 15.3 14.7 14.2 13.8 13.3 13 12.6];

% Thruster 1 ok
x13 = sqrt((Thruster.T1(1)-Thruster.T3(1))^2+(Thruster.T1(2)-Thruster.T3(2))^2);
Ratio13 = x13/D;
C13 = interp1(x,v,Ratio13);
C13 = round(C13,1);
x12 = sqrt((Thruster.T1(1)-Thruster.T2(1))^2+(Thruster.T1(2)-Thruster.T2(2))^2);
Ratio12 = x12/D;
C12 = interp1(x,v,Ratio12);
C12 = round(C12,1);
C1 = [atan2d((Thruster.T1(2)-Thruster.T3(2)),(Thruster.T1(1)-Thruster.T3(1)))+C13/2 atan2d((Thruster.T1(2)-Thruster.T3(2)),(Thruster.T1(1)-Thruster.T3(1)))-C13/2 ...
      atan2d((Thruster.T1(2)-Thruster.T2(2)),(Thruster.T1(1)-Thruster.T2(1)))-C12/2 atan2d((Thruster.T1(2)-Thruster.T2(2)),(Thruster.T1(1)-Thruster.T2(1)))+C12/2];
C1 = round(C1,1);  


% Thruster 2 ok
x21 = sqrt((Thruster.T2(1)-Thruster.T1(1))^2+(Thruster.T2(2)-Thruster.T1(2))^2);
Ratio21 = x21/D;
C21 = interp1(x,v,Ratio21);
C21 = round(C21,1);
x23 = sqrt((Thruster.T2(1)-Thruster.T3(1))^2+(Thruster.T2(2)-Thruster.T3(2))^2);
Ratio23 = x23/D;
C23 = interp1(x,v,Ratio23);
C23 = round(C23,1);
C2 = [ atan2d((Thruster.T2(2)-Thruster.T3(2)),(Thruster.T2(1)-Thruster.T3(1)))+C23/2 atan2d((Thruster.T2(2)-Thruster.T3(2)),(Thruster.T2(1)-Thruster.T3(1)))-C23/2 ...
    atan2d((Thruster.T2(2)-Thruster.T1(2)),(Thruster.T2(1)-Thruster.T1(1)))+C21/2 atan2d((Thruster.T2(2)-Thruster.T1(2)),(Thruster.T2(1)-Thruster.T1(1)))-C21/2]; 
C2 = round(C2,1);

% Thruster 3 ok
x31 = sqrt((Thruster.T3(1)-Thruster.T1(1))^2+(Thruster.T3(2)-Thruster.T1(2))^2);
Ratio31 = x31/D;
C31 = interp1(x,v,Ratio31);
C31 = round(C31,1);
x32 = sqrt((Thruster.T3(1)-Thruster.T2(1))^2+(Thruster.T3(2)-Thruster.T2(2))^2);
Ratio32 = x32/D;
C32 = interp1(x,v,Ratio32);
C32 = round(C32,1);
C3 = [atan2d((Thruster.T3(2)-Thruster.T2(2)),(Thruster.T3(1)-Thruster.T2(1)))-C32/2 atan2d((Thruster.T3(2)-Thruster.T2(2)),(Thruster.T3(1)-Thruster.T2(1)))+C32/2 ...
    atan2d((Thruster.T3(2)-Thruster.T1(2)),(Thruster.T3(1)-Thruster.T1(1)))-C31/2 atan2d((Thruster.T3(2)-Thruster.T1(2)),(Thruster.T3(1)-Thruster.T1(1)))+C31/2];
 C3 = round(C3,1);


% Thruster 4 ok
x45 = sqrt((Thruster.T4(1)-Thruster.T5(1))^2+(Thruster.T4(2)-Thruster.T5(2))^2);
Ratio45 = x45/D;
C45 = interp1(x,v,Ratio45);
C45 = round(C45,1);
x46 = sqrt((Thruster.T4(1)-Thruster.T6(1))^2+(Thruster.T4(2)-Thruster.T6(2))^2);
Ratio46 = x46/D;
C46 = interp1(x,v,Ratio46);
C46 = round(C46,1);
C4 = [atan2d((Thruster.T4(2)-Thruster.T6(2)),(Thruster.T4(1)-Thruster.T6(1)))-C46/2 atan2d((Thruster.T4(2)-Thruster.T6(2)),(Thruster.T4(1)-Thruster.T6(1)))+C46/2 ... 
    atan2d((Thruster.T4(2)-Thruster.T5(2)),(Thruster.T4(1)-Thruster.T5(1)))+C45/2 atan2d((Thruster.T4(2)-Thruster.T5(2)),(Thruster.T4(1)-Thruster.T5(1)))-C45/2];
C4 = round(C4,1);

% Thruster 5
x54 = sqrt((Thruster.T5(1)-Thruster.T4(1))^2+(Thruster.T5(2)-Thruster.T4(2))^2);
Ratio54 = x54/D;
C54 = interp1(x,v,Ratio54);
C54 = round(C54,1);
x56 = sqrt((Thruster.T5(1)-Thruster.T6(1))^2+(Thruster.T5(2)-Thruster.T6(2))^2);
Ratio56 = x56/D;
C56 = interp1(x,v,Ratio56);
C56 = round(C56,1);
C5 = [atan2d((Thruster.T5(2)-Thruster.T6(2)),(Thruster.T5(1)-Thruster.T6(1)))-C56/2 atan2d((Thruster.T5(2)-Thruster.T6(2)),(Thruster.T5(1)-Thruster.T6(1)))+C56/2 ...
    atan2d((Thruster.T5(2)-Thruster.T4(2)),(Thruster.T5(1)-Thruster.T4(1)))-C54/2 atan2d((Thruster.T5(2)-Thruster.T4(2)),(Thruster.T5(1)-Thruster.T4(1)))+C54/2];
C5 = round(C5,1);

% Thruster 6
x64 = sqrt((Thruster.T6(1)-Thruster.T4(1))^2+(Thruster.T6(2)-Thruster.T4(2))^2);
Ratio64 = x64/D;
C64 = interp1(x,v,Ratio64);
C64 = round(C64,1);
x65 = sqrt((Thruster.T6(1)-Thruster.T5(1))^2+(Thruster.T6(2)-Thruster.T5(2))^2);
Ratio65 = x65/D;
C65 = interp1(x,v,Ratio65);
C65 = round(C65,1);
C6 = [atan2d((Thruster.T6(2)-Thruster.T5(2)),(Thruster.T6(1)-Thruster.T5(1)))+C65/2 atan2d((Thruster.T6(2)-Thruster.T5(2)),(Thruster.T6(1)-Thruster.T5(1)))-C65/2 ...
      atan2d((Thruster.T6(2)-Thruster.T4(2)),(Thruster.T6(1)-Thruster.T4(1)))+C64/2 atan2d((Thruster.T6(2)-Thruster.T4(2)),(Thruster.T6(1)-Thruster.T4(1)))-C64/2];
C6 = round(C6,1);

C = [C1;C2;C3;C4;C5;C6]; % All constraints for Thruster-Thruster Interaction
