%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% run_ES_simplified.m
%
% Project:	"Autonomous heading control in position mooring with thrust 
%           assist"
%
% Abstract:	This script runs a verification of a sinusoidal perturbation 
%           based extremum seeking controller. The controller is tested
%           for a simplified 1 DOF yaw system.
%           
% Author:	Silje Aarvik Johannessen, Spring 2017
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

close all;
clear all;
clc;

%% Extremum seeking
w = (2*pi)/1700;
wl = 0.2*w;
wh = 0.1*w;
k = 0.5;
a = 0.04;

%% Environmental loads
M = 0.05;
psi_star = 20*pi/180;

%% Filtering of the psi_d
On = 900;
Off = 1;
Ts = Off; % Where on is with filtering and off is without

%% Simulation
tend = 50000;
sim ES_simplified

%% Plot

width2=7.8;
height2=6.8;
f2 = figure('Units','centimeters','Position',[0,0,width2,height2],'PaperPositionMode','auto');
plot(psi.Time,psi.Data*180/pi,psi_ref.Time,psi_ref.Data*180/pi,[0 tend], [psi_star psi_star]*180/pi, 'k--');
grid on;
legend('\psi','\psi_{d}','\psi^{*}','location', 'southeast');
xlabel('Time [s]','interpreter','latex');
ylabel('$\psi$ [deg]','interpreter','latex');
%title('Heading','Interpreter','latex')
set(gca, 'TickLabelInterpreter','latex');
set(f2, 'PaperSize',[width2, height2]);
print(f2, 'heading_simpES', '-dpdf');

f3 = figure('Units','centimeters','Position',[0,0,width2,height2],'PaperPositionMode','auto');
plot(J_psi_d.Time,J_psi_d.Data);
grid on;
%axis([0 tend -0.1 0.01]);
xlabel('Time [s]','interpreter','latex');
ylabel('y','interpreter','latex');
%title('$Performance function$','Interpreter','latex')
set(gca, 'TickLabelInterpreter','latex');
set(f3, 'PaperSize',[width2, height2]);
print(f3, 'y_simpES', '-dpdf');

f4 = figure('Units','centimeters','Position',[0,0,width2,height2],'PaperPositionMode','auto');
plot(tau_yaw.Time,tau_yaw.Data);
grid on;
%axis([0 tend -0.03 0.01]);
xlabel('Time [s]','interpreter','latex');
ylabel('$\tau_{yaw}$ [Nm]','interpreter','latex');
%title('Control moment in yaw','Interpreter','latex')
set(gca, 'TickLabelInterpreter','latex');
set(f4, 'PaperSize',[width2, height2]);
print(f4, 'tau_simpES', '-dpdf');

f5 = figure('Units','centimeters','Position',[0,0,width2,height2],'PaperPositionMode','auto');
plot(bias.Time,bias.Data);
grid on;
%axis([0 tend -0.005 0.025]);
xlabel('Time [s]','interpreter','latex');
ylabel('$\tau_{env} [Nm]$','interpreter','latex');
%title('Environmental loads','Interpreter','latex');
set(gca, 'TickLabelInterpreter','latex');
set(f5, 'PaperSize',[width2, height2]);
print(f5, 'env_simpES', '-dpdf');
